# TODO: Add comment
# 
# Author: frohlich
###############################################################################


# node: vector = genes of interest (Entrez gene IDs)
# D: shortest path distance between nodes in original graph G
# accessible.r: list of nodes from which there exists path to a given node
# path.dist.cutoff: maximal allowed pathway distance for an LCA
# output: lowest common ancestors and their shortest path distances to genes of interest
getLowestCommonAncestors = function(node, D, accessible.r, path.dist.cutoff=2){		
	common = unique(c(node[1], accessible.r[[node[1]]]))
	if(length(node) > 1){
		for(n in node[2:length(node)]){
			common = intersect(common, c(n, accessible.r[[n]]))
		}
	}
	if(length(common) == 0)
		return(list(lca=NULL, lca.dist=Inf))
	
	DD = D[common, node, drop=F]	
	DD = DD[apply(DD, 1, function(x) all(x <= path.dist.cutoff)), ,drop=F]	# k-CA
	if(NROW(DD) == 0)
		return(list(lca=NULL, lca.dist=Inf))
	dis = apply(DD, 1, sum)		# sum or average does not matter
	lca = rownames(DD)[dis == min(dis)] 
	if(length(lca) > 0)
		lca.dist = max(D[lca, node, drop=F])
	else
		return(list(lca=NULL, lca.dist=Inf))
	list(lca=lca, lca.dist=lca.dist)		
}

#	load("/home/abidata/datasets/ProteinInteractions/PathwayCommons_graph.rda")

#INPUT:
# D, DU: directed and undirected shortest path distance matrices
# signature.list: list of character vectors of Entrez gene IDs
# max.pathway.dist: maximal pathway distance, which is used to cut the clustering dendrogram
compare.signatures = function(D, DU, signature.list, max.pathway.dist=6){			
	freq = table(unlist(signature.list))	
	overlap = names(freq)[freq > length(signature.list)/2] # genes appearing in the majority of signatures are assumed to be in the consensus
	cat("number of genes appearing in > 50% of signatures: ", length(overlap), "\n")					
	candidates = intersect(setdiff(unlist(signature.list), overlap), colnames(DU))
	accessible.r = lapply(candidates, function(can) rownames(D)[D[,can] != Inf & D[,can] > 0]) # all nodes, which can access a specific candidate (= all anchestors)
	names(accessible.r) = candidates
	if(length(candidates) == 0)
		return(list(consensus=overlap, lca.path.dists=NULL))
	DD = DU[candidates, candidates]
	DD[DD == Inf] = max(DD[DD!= Inf]) + 1	
	hc = hclust(as.dist(DD), method="complete")
	consensus = NULL		
	lcas = list()
	min.fraction = 1  # minimal number of signatures that have to be covered by one LCA
	for(p in seq(1, max.pathway.dist, by=1)){
		clust = cutree(hc, h=p)		
		freq = table(clust)
		clusters = freq[freq > min.fraction]
		myclusters = NULL
		for(cl in names(clusters)){
			cl.genes = names(clust)[clust == cl]
			cl.genes.per.sig = sapply(signature.list, function(s) sum(cl.genes %in% s))
			if(sum(cl.genes.per.sig > 0) > min.fraction)
				myclusters = c(myclusters, cl)
		}
		if(length(myclusters) > 0){
			mylcas = sapply(myclusters, function(cl) getLowestCommonAncestors(names(clust)[clust == as.numeric(cl)], D, accessible.r))
			lcas[[p]] = mylcas			
			consensus = union(consensus, union(unlist(mylcas[1,]), overlap))			
		}
		else{
			lcas[[p]] = NULL
			consensus = union(consensus, overlap)
		}				
	}		
	consensus
}

